;===============================================================================================
;
; Oster-Contest: Servo (-Tester) 3 oder Voltmeter fr Schwersichtige
;
;===============================================================================================
;
;                  ATtiny13                
;                _____   _____  
;               |     |_|     |          1N4148    /
;               |Res       Vcc|-------+---|<|---+-o o-o +4,5V-6V
;               |             |       |         |
;               |             |      --- C1     |
;               |          sck|      --- 1u     |
;         +-----|PB3       PB2|-----+ |         |              
;         |     |             |     | |         |
;         |     |             |     | |         |  
;         |     |         miso|     | |         |    
;         | +---|ADC2      PB1|---+ | |         |
;         | |   |             |   | | |         |
;         | |   |             |   | | |         |
;         | |   |         mosi|   | | |         |
;         | | +-|Gnd       PB0|-+ | | |         |
;         | | | |_____________| | | | |         |
;         | | o--o--o--o--o--o--|-|-|-+ schwarz |
;         | | o--o--o--o--o--o--|-|-|-----------+ rot
;         | +-o  o  o  o  o-----+ | |   gelb
;         +------+  |  +----------+ |
;                   +---------------+
;            B4 B3 B2 B1 B0 Batt
;
;
; ALLE GND-Anschlsse MSSEN (sternfrmig) auf krzestem Weg am Pin 4 zusammenlaufen, 
; um keine Spannungsabflle auf ADC(2) zu bertragen.
;
; Die Skala wird mit einem DVM geeicht
;===============================================================================================
; ADC(2)=PB.4 wird per Interrupt in r16 geholt. Uref auf 1,1V eingestellt
; ist ADC=255 wird die Clipping-LED=PB.2 eingeschaltet
; 
; Zuerst wird r18 mit dem Startwert fr Rechtsanschlag geladen und bis 0 herunter gezhlt (main0) ~ 600us
;
; dann wird r18 herunter gezhlt (255-0) und mit r16(adc) verglichen, 
; bei bereinstimmung wird PB.1 auf 0 gesetzt, Pausenanfang (main1+2) ~ 2000us
; # durch hinzufgen/entfernen eines NOP wird der Gesamtwinkel (165) bestimmt. 
;
; nun sind ~ 2,6ms vergangen und die Rest-Pause beginnt. (main3) ~ 17400us
; nur in main3 wird der ADC-Int erlaubt, um das zappeln zu verringern. 
; ggf. kann ein 100nF am ADC gegen GND das zappeln weiter verringern. 
;===============================================================================================
; das Ganze funktioniert nur bei 1,2MHz
; Fuse 0b0110_1010
;    devide=^ ^^^^=9,6MHz 
;===============================================================================================
.DEVICE ATtiny13		; fr gavrasm
.cseg				; codesegment
.org 0				; adr 0
	rjmp init		; Reset vector
	reti 			; Int0 interrupt vector
	reti	 		; PCINT0 vector
	reti 			; TC0 overflow vector
	reti			; Eeprom ready vector
	reti			; Analog comparator int vector
	reti			; TC0 CompA vector
	reti			; TC0 CompB vector
	reti			; WDT vector
	rjmp int_adc		; ADC conversion complete vector
;--------------------------------
Int_adc:			; Aufruf NUR, wenn neuer Wert verfgbar
	in r16,ADCh		; Lese ADC-Wert, obere 8Bit
	sbi portb,3		; Clip on
	cpse r16,r17		; skip if adc=r17 ?		(1Cy)
	cbi portb,3		; Clip off
	reti			; zurck ins Hauptprogramm
;--------------------------------
init:	ldi r16,0b00001010	; Ausgaang definieren
	out ddrb,r16		; 
	ldi r17,255		; clipping-Wert
init_adc:			;
	ldi r16,0b01100010	;   - 1 1 -  - - 1 0
	Out Admux , R16		; ref-^ ^adlar   ^+^-adc2
	ldi r16,0b00000000	;   - - - -  - 0 0 0
	Out Adcsrb , R16	;              ^+^+^-Free Run
	ldi r16,0b11101110	;     1 1 1 0  1 1 1 1
	Out Adcsra , R16	; ena-^ ^-start  ^+^+^-prescale
;--------------------------------
main:	cli			; verbiete Interrupts		(1Cy) nur einmal
	sbi portb,1		; Pulsanfang			(1Cy) nur einmal
	ldi r18,208		; offset k (rechts)		(1Cy) nur einmal
main0:	dec r18			; 0->255->0			(1Cy)
	brbc 1,main0		; springe wenn Z=0		(2Cy) us ~ k * 3 Cy
;--------------------------------			
main1:	dec r18			; fang mit 255 an ->0		(1Cy)
	cpse r16,r18		; skip if R18=adc ?		(1Cy)
	rjmp main2		; nein, weiterzhlen		(2Cy)
	cbi portb,1		; ja, Pulsende			(2Cy) nur einmal
main2:	nop			; # 165-Anpassung #		(1Cy)
	nop			; # Winkel #			(1Cy)
	brbc 1,main1		; Puls+Pause 			(2Cy) us ~ 256 * 8Cy
;--------------------------------			
	sei			; erlaube adc-Int		(1Cy) nur einmal
	ldi r25,17		; pause k		 	(1Cy) nur einmal
main3:	sbiw r24,1		; dec				(2Cy)
	brbc 1,main3		; springe wenn Z=0		(2Cy) us ~ 256 * k * 4Cy
	rjmp main		; nochmal			(2Cy) nur einmal
;===============================================================================================
;end
